﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;
using Windows.Graphics.Imaging;
using Windows.Media.FaceAnalysis;
using Windows.Storage.Streams;

namespace TestProject
{
    /// <summary>
    /// 얼굴 탐지 서비스
    /// </summary>
    public class FaceDetectionService : IFaceDetectionService
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 탐지 얼굴 리스트 구하기 - GetDetectedFaceList(stream)

        /// <summary>
        /// 탐지 얼굴 리스트 구하기
        /// </summary>
        /// <param name="stream">스트림</param>
        /// <returns>탐지 얼굴 리스트 태스크</returns>
        public async Task<IList<DetectedFace>> GetDetectedFaceList(Stream stream)
        {
            IRandomAccessStream randomAccessStream = stream.AsRandomAccessStream();

            var bitmapDecoder = await BitmapDecoder.CreateAsync(randomAccessStream);

            using SoftwareBitmap sourceBitmap = await bitmapDecoder.GetSoftwareBitmapAsync();

            SoftwareBitmap targetBitmap = FaceDetector.IsBitmapPixelFormatSupported(sourceBitmap.BitmapPixelFormat)
                ? sourceBitmap : SoftwareBitmap.Convert(sourceBitmap, BitmapPixelFormat.Gray8);

            FaceDetector faceDetector = await FaceDetector.CreateAsync();

            IList<DetectedFace> detectedFaceList = await faceDetector.DetectFacesAsync(targetBitmap);

            return detectedFaceList;
        }

        #endregion
        #region 탐지 얼굴 비트맵 구하기 - GetDetectedFaceBitmap(stream, detectedFaceList, boxColor, strokeThickness)

        /// <summary>
        /// 탐지 얼굴 비트맵 구하기
        /// </summary>
        /// <param name="stream"></param>
        /// <param name="detectedFaceList"></param>
        /// <param name="boxColor"></param>
        /// <param name="strokeThickness"></param>
        /// <returns></returns>
        public System.Drawing.Bitmap GetDetectedFaceBitmap
        (
            Stream               stream,
            IList<DetectedFace>  detectedFaceList,
            System.Drawing.Color boxColor,
            int                  strokeThickness = 2
        )
        {
            System.Drawing.Bitmap bitmap = new System.Drawing.Bitmap(stream);
            
            using(System.Drawing.Graphics graphics = System.Drawing.Graphics.FromImage(bitmap))
            {
                using System.Drawing.Pen pen = new System.Drawing.Pen(boxColor, strokeThickness);

                foreach(DetectedFace detectedFace in detectedFaceList)
                {
                    BitmapBounds bitmapBounds = detectedFace.FaceBox;

                    graphics.DrawRectangle
                    (
                        pen,
                        (int)bitmapBounds.X,
                        (int)bitmapBounds.Y,
                        (int)bitmapBounds.Width,
                        (int)bitmapBounds.Height
                    );
                }
            }

            return bitmap;
        }

        #endregion
    }
}