﻿using System.IO;
using System.Windows;
using System.Windows.Media.Imaging;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            Loaded += Window_Loaded;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 윈도우 로드시 처리하기 - Window_Loaded(sender, e)

        /// <summary>
        /// 윈도우 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void Window_Loaded(object sender, RoutedEventArgs e)
        {
            FaceDetectionService service = new FaceDetectionService();

            await using FileStream fileStream = File.OpenRead(@"IMAGE\sample.jpg");

            var faces = await service.GetDetectedFaceList(fileStream);

            System.Drawing.Bitmap FacesBitmap = service.GetDetectedFaceBitmap(fileStream, faces, System.Drawing.Color.Red, 8);

            this.image.Source = GetBitmapImage(FacesBitmap);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 비트맵 이미지 구하기 - GetBitmapImage(bitmap)

        /// <summary>
        /// 비트맵 이미지 구하기
        /// </summary>
        /// <param name="bitmap">비트맵</param>
        /// <returns>비트맵 이미지</returns>
        private BitmapImage GetBitmapImage(System.Drawing.Bitmap bitmap)
        {
            using MemoryStream stream = new MemoryStream();

            bitmap.Save(stream, System.Drawing.Imaging.ImageFormat.Bmp);

            stream.Position = 0;

            BitmapImage bitmapImage = new BitmapImage();

            bitmapImage.BeginInit();

            bitmapImage.CacheOption  = BitmapCacheOption.OnLoad;
            bitmapImage.StreamSource = stream;

            bitmapImage.EndInit();

            bitmapImage.Freeze();

            return bitmapImage;
        }

        #endregion
    }
}